'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _scanner = require('./scanner');

var _scanner2 = _interopRequireDefault(_scanner);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @file Parser
 */

var Parser = function () {

  /**
   * @brief Constructor
   *
   * @param template   Template needs to parse
   */

  function Parser(template) {
    _classCallCheck(this, Parser);

    this.template = template;
  }

  /**
   * @brief Transform template string to token stream.
   */


  _createClass(Parser, [{
    key: 'tokens',
    value: function tokens(template) {
      var scanner = new _scanner2.default(template);

      var tokens = [];

      if (!template) {
        return [];
      }

      var openingTagRe = /\{\{\s*/,
          closingTagRe = /\s*\}\}/,
          closingCurlyRe = /\s*\}\}/,
          tagRe = /#|\{|\^|\!|\//;

      var start = void 0,
          type = void 0,
          value = void 0,
          ch = void 0,
          hasTag = void 0,
          token = void 0;
      while (!scanner.eot()) {
        start = scanner.pos;
        value = scanner.scanUntil(openingTagRe);
        for (var i = 0; i < value.length; i++) {
          ch = value.charAt(i);
          tokens.push(['text', ch, start, start + 1]);
          start += 1;
        }

        // Un match opening tag.
        if (!scanner.scan(openingTagRe)) {
          break;
        }
        start = scanner.pos;

        hasTag = true;

        type = scanner.scan(tagRe) || 'name';

        // Variable condition
        if (type === '{') {
          value = scanner.scanUntil(closingCurlyRe);
          scanner.scan(closingCurlyRe);
          scanner.scanUntil(closingTagRe);
          type = '&';
        } else {
          value = scanner.scanUntil(closingTagRe);
        }

        // Ensure each opening tag has closing tag as a pair.
        if (!scanner.scan(closingTagRe)) throw new Error('Unclosed tag at ' + scanner.pos);

        token = [type, value, start, scanner.pos];
        tokens.push(token);
      }
      return tokens;
    }

    /**
     * @brief Combines the values of consecutive text tokens in the given `tokens` array to a single token.
     * @param tokens   Token stream array
       */

  }, {
    key: 'squashTokens',
    value: function squashTokens(tokens) {
      var squashedTokens = [];

      var token = void 0,
          lastToken = void 0;
      for (var i = 0, len = tokens.length; i < len; i++) {
        token = tokens[i];

        if (token) {
          if (token[0] === 'text' && lastToken && lastToken[0] === 'text') {
            // Merge text content
            lastToken[1] += token[1];
            // Update end indicator
            lastToken[3] = token[3];
          } else {
            squashedTokens.push(token);
            lastToken = token;
          }
        }
      }

      return squashedTokens;
    }

    /**
     * @brief Transform single tokens to nested tree structure  where
     * tokens that represent a section have two additional items: 1) an array of
     * all tokens that appear in that section and 2) the index in the original
     * template that represents the end of that section.
     *
     * @param tokens
     */

  }, {
    key: 'nestedTokens',
    value: function nestedTokens(tokens) {
      var nestedTokens = [];
      var collector = nestedTokens;
      var sections = [];

      var token, section;
      for (var i = 0, numTokens = tokens.length; i < numTokens; ++i) {
        token = tokens[i];

        switch (token[0]) {
          case '#':
            collector.push(token);
            sections.push(token);
            collector = token[4] = [];
            break;
          case '/':
            section = sections.pop();
            section[5] = token[2];
            collector = sections.length > 0 ? sections[sections.length - 1][4] : nestedTokens;
            break;
          default:
            collector.push(token);
        }
      }

      return nestedTokens;
    }
  }, {
    key: 'parse',
    value: function parse() {
      var template = this.template;
      var tokens = this.tokens(template);
      var squashTokens = this.squashTokens(tokens);

      return this.nestedTokens(squashTokens);
    }
  }]);

  return Parser;
}();

exports.default = Parser;