'use strict';

/**
 * @file String scanner that is used by the template parser to find tokens in template strings.
 */

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Scanner = function () {

  /**
   * @brief Constructor
   *
   * @param template   Template needs to scanner
   */

  function Scanner(template) {
    _classCallCheck(this, Scanner);

    this.pos = 0;
    this.tail = template;
    this.template = template;
  }

  /**
   * @brief Means ends of template, return `true` if template traversal is ended.
   *
   * @returns {boolean}
   */


  _createClass(Scanner, [{
    key: 'eot',
    value: function eot() {
      return this.tail === '';
    }

    /**
     * @brief Try to match the given regex expression from current position.
     * @param re Regex
     * @returns {*} Return matched text if it can be matched, otherwise empty string.
       */

  }, {
    key: 'scan',
    value: function scan(re) {
      var match = this.tail.match(re);

      if (!match || match.index !== 0) return '';

      var str = match[0];
      var len = str.length;

      /* Update pos and tail. */
      this.tail = this.tail.substring(len);
      this.pos += len;

      return str;
    }

    /**
     * @brief
     * @param re
     */

  }, {
    key: 'scanUntil',
    value: function scanUntil(re) {
      var index = this.tail.search(re);
      var match = void 0;

      switch (index) {
        case -1:
          // No find
          match = this.tail;
          this.tail = '';
          break;
        case 0:
          // Has found, and index in first position
          match = '';
          break;
        default:
          // Has found, and not in first position
          match = this.tail.substring(0, index);
          this.tail = this.tail.substring(index);
      }

      this.pos += index;
      return match;
    }
  }]);

  return Scanner;
}();

exports.default = Scanner;